package com.chelvc.framework.base.context;

import java.net.URI;
import java.util.Arrays;
import java.util.Map;
import java.util.function.Function;

import com.chelvc.framework.common.util.ErrorUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.NoHttpResponseException;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.web.client.RestTemplate;

/**
 * Rest上下文工具类
 *
 * @author Woody
 * @date 2024/6/5
 */
@Slf4j
public final class RestContextHolder {
    /**
     * RestTemplate实例
     */
    private static volatile RestTemplate TEMPLATE;

    private RestContextHolder() {
    }

    /**
     * 获取RestTemplate实例
     *
     * @return RestTemplate实例
     */
    public static RestTemplate getTemplate() {
        if (TEMPLATE == null) {
            synchronized (RestTemplate.class) {
                if (TEMPLATE == null) {
                    TEMPLATE = ApplicationContextHolder.getBean(RestTemplate.class);
                }
            }
        }
        return TEMPLATE;
    }

    /**
     * 执行接口请求
     *
     * @param function 请求回调函数
     * @param <T>      结果类型
     * @return 请求结果
     */
    public static <T> T execute(@NonNull Function<RestTemplate, T> function) {
        try {
            return function.apply(getTemplate());
        } catch (Exception e) {
            if (ErrorUtils.contains(e, NoHttpResponseException.class)) {
                log.warn("Rest invoke failed and retry later: {}", e.getMessage());
                // 请求重试
                return function.apply(new RestTemplate());
            }
            throw e;
        }
    }

    /**
     * GET方式请求
     *
     * @param url  资源地址
     * @param type 对象类型
     * @param <T>  结果类型
     * @return 请求结果
     */
    public static <T> T get(@NonNull URI url, @NonNull Class<T> type) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}", url);
            }
            T response = rest.getForObject(url, type);
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * GET方式请求
     *
     * @param url        资源地址
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T get(@NonNull String url, @NonNull Class<T> type, @NonNull Object... parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}, {}", url, Arrays.toString(parameters));
            }
            T response = rest.getForObject(url, type, parameters);
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * GET方式请求
     *
     * @param url        资源地址
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T get(@NonNull String url, @NonNull Class<T> type, @NonNull Map<String, ?> parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}, {}", url, parameters);
            }
            T response = rest.getForObject(url, type, parameters);
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * POST方式请求
     *
     * @param url     资源地址
     * @param request 请求实体
     * @param type    对象类型
     * @param <T>     结果类型
     * @return 请求结果
     */
    public static <T> T post(@NonNull URI url, Object request, @NonNull Class<T> type) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}, {}", url, request);
            }
            T response = rest.postForObject(url, request, type);
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * POST方式请求
     *
     * @param url        资源地址
     * @param request    请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T post(@NonNull String url, Object request, @NonNull Class<T> type,
                             @NonNull Object... parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}, {}, {}", url, request, Arrays.toString(parameters));
            }
            T response = rest.postForObject(url, request, type, parameters);
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * POST方式请求
     *
     * @param url        资源地址
     * @param request    请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T post(@NonNull String url, Object request, @NonNull Class<T> type,
                             @NonNull Map<String, ?> parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}, {}, {}", url, request, parameters);
            }
            T response = rest.postForObject(url, request, type, parameters);
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param entity 请求实体
     * @param type   对象类型
     * @param <T>    结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull RequestEntity<?> entity, @NonNull Class<T> type) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}", entity.getUrl());
            }
            T response = rest.exchange(entity, type).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param entity 请求实体
     * @param type   对象类型
     * @param <T>    结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull RequestEntity<?> entity, @NonNull ParameterizedTypeReference<T> type) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug("Rest invoke: {}", entity.getUrl());
            }
            T response = rest.exchange(entity, type).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param url    资源地址
     * @param method 请求方式
     * @param entity 请求实体
     * @param type   对象类型
     * @param <T>    结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull URI url, @NonNull HttpMethod method, HttpEntity<?> entity,
                                 @NonNull Class<T> type) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                Object body = ObjectUtils.ifNull(entity, HttpEntity::getBody);
                log.debug("Rest invoke: {}, {}", url, body);
            }
            T response = rest.exchange(url, method, entity, type).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull String url, @NonNull HttpMethod method, HttpEntity<?> entity,
                                 @NonNull Class<T> type, @NonNull Object... parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                Object body = ObjectUtils.ifNull(entity, HttpEntity::getBody);
                log.debug("Rest invoke: {}, {}, {}", url, body, Arrays.toString(parameters));
            }
            T response = rest.exchange(url, method, entity, type, parameters).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull String url, @NonNull HttpMethod method, HttpEntity<?> entity,
                                 @NonNull Class<T> type, @NonNull Map<String, ?> parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                Object body = ObjectUtils.ifNull(entity, HttpEntity::getBody);
                log.debug("Rest invoke: {}, {}, {}", url, body, parameters);
            }
            T response = rest.exchange(url, method, entity, type, parameters).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param url    资源地址
     * @param method 请求方式
     * @param entity 请求实体
     * @param type   对象类型
     * @param <T>    结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull URI url, @NonNull HttpMethod method, HttpEntity<?> entity,
                                 @NonNull ParameterizedTypeReference<T> type) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                Object body = ObjectUtils.ifNull(entity, HttpEntity::getBody);
                log.debug("Rest invoke: {}, {}", url, body);
            }
            T response = rest.exchange(url, method, entity, type).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull String url, @NonNull HttpMethod method, HttpEntity<?> entity,
                                 @NonNull ParameterizedTypeReference<T> type, @NonNull Object... parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                Object body = ObjectUtils.ifNull(entity, HttpEntity::getBody);
                log.debug("Rest invoke: {}, {}, {}", url, body, Arrays.toString(parameters));
            }
            T response = rest.exchange(url, method, entity, type, parameters).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 请求结果
     */
    public static <T> T exchange(@NonNull String url, @NonNull HttpMethod method, HttpEntity<?> entity,
                                 @NonNull ParameterizedTypeReference<T> type, @NonNull Map<String, ?> parameters) {
        return execute(rest -> {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                Object body = ObjectUtils.ifNull(entity, HttpEntity::getBody);
                log.debug("Rest invoke: {}, {}, {}", url, body, parameters);
            }
            T response = rest.exchange(url, method, entity, type, parameters).getBody();
            if (debug) {
                log.debug("Rest response: {}", response);
            }
            return response;
        });
    }
}
