package com.chelvc.framework.common.util;

import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.Map;
import java.util.function.Supplier;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import com.chelvc.framework.common.model.Pool;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

/**
 * AES加解密工具类
 *
 * @author woody
 * @date 2024/1/30
 */
public final class AESUtils {
    /**
     * 密钥长度
     */
    public static final int KEY_SIZE = 128;

    /**
     * AES算法名称
     */
    public static final String ALGORITHM = "AES";

    /**
     * CBC NoPadding模式
     */
    public static final String CBC_NOPADDING = "AES/CBC/NoPadding";

    /**
     * CBC PKCS5Padding模式
     */
    public static final String CBC_PKCS5PADDING = "AES/CBC/PKCS5Padding";

    /**
     * ECB PKCS5Padding模式
     */
    public static final String ECB_PKCS5PADDING = "AES/ECB/PKCS5Padding";

    /**
     * 加密处理器名称/对象池映射表
     */
    private static final Map<String, Pool<Cipher>> ENCRYPTOR_CACHE = Maps.newConcurrentMap();

    /**
     * 解密处理器名称/对象池映射表
     */
    private static final Map<String, Pool<Cipher>> DECRYPTER_CACHE = Maps.newConcurrentMap();

    static {
        Security.addProvider(new BouncyCastleProvider());
        try {
            KeyGenerator.getInstance(ALGORITHM).init(KEY_SIZE);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    private AESUtils() {
    }

    /**
     * 将密钥转换成向量
     *
     * @param secret 密钥
     * @return 向量
     */
    public static String secret2iv(@NonNull String secret) {
        int size = Math.min(secret.length(), 16);
        if (size == 0) {
            return StringUtils.EMPTY;
        }
        StringBuilder iv = new StringBuilder(size);
        for (int i = secret.length() - 1, max = 0; i >= 0 && max < size; i--, max++) {
            iv.append(secret.charAt(i));
        }
        return iv.toString();
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull String secret) {
        return getCipher(mode, secret.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull byte[] secret) {
        return getCipher(ECB_PKCS5PADDING, mode, secret);
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull String secret) {
        return getCipher(name, mode, secret.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull byte[] secret) {
        try {
            Cipher cipher = Cipher.getInstance(name);
            cipher.init(mode, new SecretKeySpec(secret, ALGORITHM));
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull String secret, @NonNull String iv) {
        return getCipher(mode, secret.getBytes(StandardCharsets.UTF_8), iv.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull byte[] secret, @NonNull byte[] iv) {
        return getCipher(CBC_PKCS5PADDING, mode, secret, iv);
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull String secret, @NonNull String iv) {
        return getCipher(name, mode, secret.getBytes(StandardCharsets.UTF_8), iv.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 获取密码处理器
     *
     * @param name   处理器名称
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(@NonNull String name, int mode, @NonNull byte[] secret, @NonNull byte[] iv) {
        try {
            Cipher cipher = Cipher.getInstance(name);
            Key key = new SecretKeySpec(secret, ALGORITHM);
            cipher.init(mode, key, new IvParameterSpec(iv));
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 从缓存中查找密码处理器，如果不存在则新建
     *
     * @param key      缓存标识
     * @param mode     加解密模式
     * @param supplier 处理器实例构建回调函数
     * @return 处理器实例
     */
    public static Cipher lookupCipher(@NonNull String key, int mode, @NonNull Supplier<Cipher> supplier) {
        Map<String, Pool<Cipher>> cache;
        if (mode == Cipher.ENCRYPT_MODE) {
            cache = ENCRYPTOR_CACHE;
        } else if (mode == Cipher.DECRYPT_MODE) {
            cache = DECRYPTER_CACHE;
        } else {
            throw new IllegalArgumentException("Not support cipher mode: " + mode);
        }
        return cache.computeIfAbsent(key, k -> new Pool<>(supplier)).get();
    }

    /**
     * 获取加密处理器
     *
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull String secret) {
        return getCipher(Cipher.ENCRYPT_MODE, secret);
    }

    /**
     * 获取加密处理器
     *
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull byte[] secret) {
        return getCipher(Cipher.ENCRYPT_MODE, secret);
    }

    /**
     * 获取加密处理器
     *
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull String secret, @NonNull String iv) {
        return getCipher(Cipher.ENCRYPT_MODE, secret, iv);
    }

    /**
     * 获取加密处理器
     *
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getEncryptor(@NonNull byte[] secret, @NonNull byte[] iv) {
        return getCipher(Cipher.ENCRYPT_MODE, secret, iv);
    }

    /**
     * 从缓存中查找加密处理器，如果不存在则新建
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher lookupEncryptor(@NonNull String name, @NonNull String secret) {
        return lookupEncryptor(name + secret, () -> getCipher(name, Cipher.ENCRYPT_MODE, secret));
    }

    /**
     * 从缓存中查找加密处理器，如果不存在则新建
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher lookupEncryptor(@NonNull String name, @NonNull String secret, @NonNull String iv) {
        return lookupEncryptor(name + secret + iv, () -> getCipher(name, Cipher.ENCRYPT_MODE, secret, iv));
    }

    /**
     * 从缓存中查找加密处理器，如果不存在则新建
     *
     * @param key      缓存标识
     * @param supplier 处理器实例构建回调函数
     * @return 处理器实例
     */
    public static Cipher lookupEncryptor(@NonNull String key, @NonNull Supplier<Cipher> supplier) {
        return lookupCipher(key, Cipher.ENCRYPT_MODE, supplier);
    }

    /**
     * 获取解密处理器
     *
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull String secret) {
        return getCipher(Cipher.DECRYPT_MODE, secret);
    }

    /**
     * 获取解密处理器
     *
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull byte[] secret) {
        return getCipher(Cipher.DECRYPT_MODE, secret);
    }

    /**
     * 获取解密处理器
     *
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull String secret, @NonNull String iv) {
        return getCipher(Cipher.DECRYPT_MODE, secret, iv);
    }

    /**
     * 获取解密处理器
     *
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher getDecrypter(@NonNull byte[] secret, @NonNull byte[] iv) {
        return getCipher(Cipher.DECRYPT_MODE, secret, iv);
    }

    /**
     * 从缓存中查找解密处理器，如果不存在则新建
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @return 处理器实例
     */
    public static Cipher lookupDecrypter(@NonNull String name, @NonNull String secret) {
        return lookupDecrypter(name + secret, () -> getCipher(name, Cipher.DECRYPT_MODE, secret));
    }

    /**
     * 从缓存中查找解密处理器，如果不存在则新建
     *
     * @param name   处理器名称
     * @param secret 密钥
     * @param iv     向量
     * @return 处理器实例
     */
    public static Cipher lookupDecrypter(@NonNull String name, @NonNull String secret, @NonNull String iv) {
        return lookupDecrypter(name + secret + iv, () -> getCipher(name, Cipher.DECRYPT_MODE, secret, iv));
    }

    /**
     * 从缓存中查找解密处理器，如果不存在则新建
     *
     * @param key      缓存标识
     * @param supplier 处理器实例构建回调函数
     * @return 处理器实例
     */
    public static Cipher lookupDecrypter(@NonNull String key, @NonNull Supplier<Cipher> supplier) {
        return lookupCipher(key, Cipher.DECRYPT_MODE, supplier);
    }
}
