package com.chelvc.framework.database.support;

import java.io.Serializable;
import java.util.Collections;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.NonNull;

/**
 * 自定义字段更新工具类
 *
 * @author Woody
 * @date 2024/5/18
 */
public final class Updates {
    private Updates() {
    }

    /**
     * 获取字段更新对象
     *
     * @param <T> 数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> update() {
        return new Update<>();
    }

    /**
     * 更新所有字段
     *
     * @param clazz 对象类型
     * @param <T>   数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> all(@NonNull Class<T> clazz) {
        return not(clazz, StringUtils.EMPTY_ARRAY);
    }

    /**
     * 更新除指定字段外的所有字段
     *
     * @param clazz   对象类型
     * @param columns 排除字段数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> not(@NonNull Class<T> clazz, @NonNull String... columns) {
        Update<T> update = update();
        Map<String, ColumnCache> properties = LambdaUtils.getColumnMap(clazz);
        if (ObjectUtils.isEmpty(properties)) {
            return update;
        }
        TableInfo table = TableInfoHelper.getTableInfo(clazz);
        String key = StringUtils.ifEmpty(table.getKeyProperty(), LambdaUtils::formatKey);
        Set<String> excludes = ObjectUtils.isEmpty(columns) ? Collections.emptySet() : Sets.newHashSet(columns);
        properties.forEach((property, column) -> {
            if (!Objects.equals(property, key) && !excludes.contains(column.getColumn())) {
                update.use(column.getColumn());
            }
        });
        return update;
    }

    /**
     * 更新除指定字段外的所有字段
     *
     * @param clazz   对象类型
     * @param getters 排除Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> not(@NonNull Class<T> clazz, @NonNull SFunction<T, ?>... getters) {
        if (ObjectUtils.isEmpty(getters)) {
            return not(clazz, StringUtils.EMPTY_ARRAY);
        }
        String[] columns = new String[getters.length];
        for (int i = 0; i < getters.length; i++) {
            columns[i] = DatabaseContextHolder.getter2column(getters[i]);
        }
        return not(clazz, columns);
    }

    /**
     * 使用
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> use(@NonNull String... columns) {
        Update<T> update = update();
        return update.use(columns);
    }

    /**
     * 使用
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> use(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.use(getters);
    }

    /**
     * 相加
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> add(@NonNull String... columns) {
        Update<T> update = update();
        return update.add(columns);
    }

    /**
     * 相加
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> add(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.add(getters);
    }

    /**
     * 清空
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> clear(@NonNull String... columns) {
        Update<T> update = update();
        return update.clear(columns);
    }

    /**
     * 清空
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> clear(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.clear(getters);
    }

    /**
     * 相减
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> subtract(@NonNull String... columns) {
        Update<T> update = update();
        return update.subtract(columns);
    }

    /**
     * 相减
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> subtract(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.subtract(getters);
    }

    /**
     * 相乘
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> multiply(@NonNull String... columns) {
        Update<T> update = update();
        return update.multiply(columns);
    }

    /**
     * 相乘
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> multiply(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.multiply(getters);
    }

    /**
     * 相除
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> divide(@NonNull String... columns) {
        Update<T> update = update();
        return update.divide(columns);
    }

    /**
     * 相除
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> divide(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.divide(getters);
    }

    /**
     * 设置更新表达式
     *
     * @param column 列名
     * @param value  值
     * @param <T>    数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> set(@NonNull String column, @NonNull String value) {
        Update<T> update = update();
        return update.set(column, value);
    }

    /**
     * 字段更新对象
     *
     * @param <T> 数据类型
     */
    public static class Update<T> implements Serializable {
        private final Map<String, String> expressions = Maps.newLinkedHashMap();

        /**
         * 使用
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> use(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, "VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 使用
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> use(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.use(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相加
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> add(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " + VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相加
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> add(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.add(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 清空
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> clear(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, "null");
            }
            return this;
        }

        /**
         * 清空
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> clear(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.clear(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相减
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> subtract(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " - VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相减
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> subtract(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.subtract(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相乘
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> multiply(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " * VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相乘
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> multiply(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.multiply(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相除
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> divide(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " / VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相除
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> divide(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.divide(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 设置更新表达式
         *
         * @param column     列名
         * @param expression 值表达式
         * @return 字段更新对象实例
         */
        public final Update<T> set(@NonNull String column, @NonNull String expression) {
            this.expressions.put(column, expression);
            return this;
        }

        /**
         * 获取更新字段/值表达式映射表
         *
         * @return 字段/值表达式映射表
         */
        public final Map<String, String> getExpressions() {
            return Collections.unmodifiableMap(this.expressions);
        }

        @Override
        public String toString() {
            return this.expressions.toString();
        }
    }
}
