package com.chelvc.framework.oauth.config;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import com.chelvc.framework.base.config.MultiserverMvcConfigurer;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.oauth.annotation.Authorize;
import com.chelvc.framework.oauth.context.OAuthContextHolder;
import com.chelvc.framework.oauth.token.DefaultSessionValidator;
import com.chelvc.framework.oauth.token.SessionValidator;
import com.chelvc.framework.oauth.token.TimestampTokenValidator;
import com.google.common.collect.Sets;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.core.DelegatingOAuth2TokenValidator;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.OAuth2Error;
import org.springframework.security.oauth2.core.OAuth2TokenValidator;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.security.oauth2.jwt.NimbusJwtDecoder;
import org.springframework.security.oauth2.server.resource.InvalidBearerTokenException;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationConverter;
import org.springframework.security.oauth2.server.resource.authentication.JwtGrantedAuthoritiesConverter;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.access.AccessDeniedHandler;
import org.springframework.security.web.authentication.logout.LogoutHandler;

/**
 * OAuth认证配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@EnableWebSecurity
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OAuthConfigurer extends WebSecurityConfigurerAdapter {
    private final OAuthProperties properties;
    private final ApplicationContext applicationContext;

    @Autowired(required = false)
    private LogoutHandler logoutHandler;

    @Bean
    @ConditionalOnMissingBean(SessionValidator.class)
    public SessionValidator sessionValidator() {
        return new DefaultSessionValidator();
    }

    @Bean
    public JwtDecoder jwtDecoder() {
        // 构建JWT解码器实例
        String secret = AssertUtils.nonempty(this.properties.getSecret(), () -> "OAuth secret is missing");
        SecretKey key = new SecretKeySpec(secret.getBytes(), AESUtils.ALGORITHM);
        NimbusJwtDecoder decoder = NimbusJwtDecoder.withSecretKey(key).build();

        // 添加自定义令牌验证器
        Collection<OAuth2TokenValidator<Jwt>> validators = Arrays.asList(
                new TimestampTokenValidator(),
                this.applicationContext.getBean(SessionValidator.class)
        );
        decoder.setJwtValidator(new DelegatingOAuth2TokenValidator<>(validators));
        return decoder;
    }

    @Bean
    public AccessDeniedHandler accessDeniedHandler() {
        return (request, response, e) -> {
            LoggingContextHolder.warn(log, request, e);
            String message = ApplicationContextHolder.getMessage("Forbidden");
            SessionContextHolder.response(response, Result.of(HttpStatus.FORBIDDEN.name(), null, message));
        };
    }

    @Bean
    public AuthenticationEntryPoint authenticationEntryPoint() {
        return (request, response, e) -> {
            LoggingContextHolder.warn(log, request, e);
            if (e instanceof InvalidBearerTokenException) {
                String message = ApplicationContextHolder.getMessage("Token.Invalid");
                SessionContextHolder.response(response, Result.of("TOKEN_INVALID", null, message));
            } else if (e instanceof OAuth2AuthenticationException) {
                OAuth2Error error = ((OAuth2AuthenticationException) e).getError();
                SessionContextHolder.response(response, Result.of(error.getErrorCode(), null, error.getDescription()));
            } else {
                String message = ApplicationContextHolder.getMessage("Unauthorized");
                SessionContextHolder.response(response, Result.of(HttpStatus.UNAUTHORIZED.name(), null, message));
            }
        };
    }

    @Bean
    public JwtAuthenticationConverter jwtAuthenticationConverter() {
        JwtAuthenticationConverter jwtAuthenticationConverter = new JwtAuthenticationConverter();
        JwtGrantedAuthoritiesConverter jwtGrantedAuthoritiesConverter = new JwtGrantedAuthoritiesConverter();
        jwtGrantedAuthoritiesConverter.setAuthorityPrefix(StringUtils.EMPTY);
        jwtGrantedAuthoritiesConverter.setAuthoritiesClaimName(OAuthContextHolder.AUTHORITIES);
        jwtAuthenticationConverter.setJwtGrantedAuthoritiesConverter(jwtGrantedAuthoritiesConverter);
        return jwtAuthenticationConverter;
    }

    @Bean
    @Override
    public AuthenticationManager authenticationManager() throws Exception {
        return super.authenticationManager();
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) {
        // 加载自定义认证方式
        this.applicationContext.getBeansOfType(AuthenticationProvider.class).forEach(
                (key, provider) -> auth.authenticationProvider(provider)
        );
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        // 基础配置
        http.httpBasic().and().cors().and().csrf().disable()
                .authorizeRequests().antMatchers(HttpMethod.OPTIONS).permitAll()
                .and().sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and().oauth2ResourceServer(server -> server.accessDeniedHandler(this.accessDeniedHandler())
                .authenticationEntryPoint(this.authenticationEntryPoint()).jwt().decoder(this.jwtDecoder())
                .jwtAuthenticationConverter(this.jwtAuthenticationConverter()));

        // 登出配置
        if (this.logoutHandler != null) {
            http.logout().addLogoutHandler(this.logoutHandler);
        }

        // 如果排除认证配置非空，则其他资源全部守保护
        if (ObjectUtils.notEmpty(this.properties.getIgnores())) {
            String[] ignores = this.properties.getIgnores().toArray(new String[0]);
            http.authorizeRequests().antMatchers(ignores).permitAll();
            http.authorizeRequests().anyRequest().authenticated();
            return;
        }

        // 判断是否启用多服务MVC配置，如果包含则获取所有服务资源对象
        boolean multiserver = this.applicationContext.containsBean(MultiserverMvcConfigurer.class.getName());
        List<Resource> resources = multiserver ?
                ApplicationContextHolder.getApplicationResources() : Collections.emptyList();

        // 将所有使用@Authorize注解并且enabled=true的接口地址加入认证
        Set<String> authenticates = Sets.newHashSet();
        ApplicationContextHolder.lookupControllers(this.applicationContext).forEach(controller -> {
            Class<?> clazz = AopProxyUtils.ultimateTargetClass(controller);

            // 如果启用多服务MVC配置则将服务名作为接口地址前缀
            String prefix = null;
            if (multiserver) {
                Resource resource = ApplicationContextHolder.lookupClassResource(clazz, resources);
                prefix = ObjectUtils.ifNull(resource, ApplicationContextHolder::getResourcePrefix);
            }

            // 遍历所有接口方法
            for (Method method : clazz.getDeclaredMethods()) {
                Authorize authorize = method.getAnnotation(Authorize.class);
                if ((authorize != null || (authorize = clazz.getAnnotation(Authorize.class)) != null)
                        && authorize.enabled()) {
                    for (String api : SpringUtils.getApis(method)) {
                        authenticates.add(HttpUtils.uri(prefix, api));
                    }
                }
            }
        });
        if (ObjectUtils.notEmpty(authenticates)) {
            http.authorizeRequests().antMatchers(authenticates.toArray(new String[0])).authenticated();
        }
        http.authorizeRequests().anyRequest().permitAll();
    }
}
