package com.chelvc.framework.oauth.context;

import java.text.ParseException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.common.function.Adapter;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Sets;
import com.nimbusds.jwt.JWT;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.JWTParser;
import lombok.NonNull;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.server.resource.web.BearerTokenResolver;
import org.springframework.security.oauth2.server.resource.web.DefaultBearerTokenResolver;

/**
 * OAuth上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class OAuthContextHolder {
    /**
     * 主体ID标识
     */
    public static final String ID = "id";

    /**
     * 令牌唯一标识
     */
    public static final String JTI = "jti";

    /**
     * 应用范围标识
     */
    public static final String SCOPE = "scope";

    /**
     * 手机号码标识
     */
    public static final String MOBILE = "mobile";

    /**
     * 客户端登陆标识
     */
    public static final String CLIENT = "client";

    /**
     * 用户账号标识
     */
    public static final String USERNAME = "user_name";

    /**
     * 用户权限标识
     */
    public static final String AUTHORITIES = "authorities";

    /**
     * 主体注册时间戳
     */
    public static final String REGISTERING = "registering";

    /**
     * Bearer令牌解析器
     */
    private static final BearerTokenResolver BEARER_TOKEN_RESOLVER = new DefaultBearerTokenResolver();

    private OAuthContextHolder() {
    }

    /**
     * 令牌转换
     *
     * @param token 令牌信息
     * @return JWT对象实例
     */
    public static JWT parse(String token) {
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JWTParser.parse(token);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 构建访问令牌标识
     *
     * @param id 主体标识
     * @return 令牌标识
     */
    public static String key(@NonNull Object id) {
        return "token:" + id;
    }

    /**
     * 获取请求令牌
     *
     * @param request Http请求对象
     * @return 令牌信息
     */
    public static String getToken(HttpServletRequest request) {
        return StringUtils.ifEmpty(ObjectUtils.ifNull(request, BEARER_TOKEN_RESOLVER::resolve), (String) null);
    }

    /**
     * 获取Jwt属性集合
     *
     * @param jwt JWT对象实例
     * @return Jwt属性集合
     */
    public static JWTClaimsSet getJwtClaims(JWT jwt) {
        if (Objects.isNull(jwt)) {
            return null;
        }
        try {
            return jwt.getJWTClaimsSet();
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取Jwt属性值
     *
     * @param jwt      JWT对象实例
     * @param function 属性获取方法
     * @param <T>      属性值类型
     * @return 属性值
     */
    public static <T> T getJwtClaim(JWT jwt, @NonNull Adapter<JWTClaimsSet, T> function) {
        JWTClaimsSet claims = ObjectUtils.ifNull(jwt, OAuthContextHolder::getJwtClaims);
        if (claims == null) {
            return null;
        }
        try {
            return function.apply(claims);
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            }
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(USERNAME), Long::parseLong);
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(JWT jwt) {
        return getJwtClaim(jwt, claims -> StringUtils.ifEmpty(claims.getStringClaim(USERNAME), Long::parseLong));
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JTI), (String) null);
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> claims.getStringClaim(JTI)), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param jwt Jwt对象
     * @return 应用范围
     */
    public static String getScope(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(SCOPE), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param jwt JWT对象
     * @return 应用范围
     */
    public static String getScope(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> claims.getStringClaim(SCOPE)), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param token OAuth认证令牌对象
     * @return 应用范围
     */
    public static String getScope(OAuth2AccessToken token) {
        Map<String, Object> additions = ObjectUtils.ifNull(token, OAuth2AccessToken::getAdditionalInformation);
        return additions == null ? null : StringUtils.ifEmpty(additions.get(SCOPE), String::valueOf);
    }

    /**
     * 是否是客户端登陆
     *
     * @param jwt Jwt对象
     * @return true/false
     */
    public static boolean isClient(Jwt jwt) {
        return jwt != null && Boolean.TRUE.equals(jwt.getClaimAsBoolean(CLIENT));
    }

    /**
     * 是否是客户端登陆
     *
     * @param jwt JWT对象
     * @return true/false
     */
    public static boolean isClient(JWT jwt) {
        return Boolean.TRUE.equals(getJwtClaim(jwt, claims -> claims.getBooleanClaim(CLIENT)));
    }

    /**
     * 是否是客户端登陆
     *
     * @param token OAuth认证令牌对象
     * @return true/false
     */
    public static boolean isClient(OAuth2AccessToken token) {
        Map<String, Object> additions = ObjectUtils.ifNull(token, OAuth2AccessToken::getAdditionalInformation);
        return additions != null && Boolean.TRUE.equals(additions.get(CLIENT));
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(JWT jwt) {
        String[] authorities = getJwtClaim(jwt, claims -> claims.getStringArrayClaim(AUTHORITIES));
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }
}
