package com.chelvc.framework.oauth.token;

import java.util.List;
import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.context.Using;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.oauth.config.OAuthProperties;
import com.chelvc.framework.oauth.context.OAuthContextHolder;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.redis.context.RedisHashHolder;
import com.chelvc.framework.redis.context.RedisUserDailyHashHolder;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.oauth2.core.OAuth2AuthenticationException;
import org.springframework.security.oauth2.core.OAuth2Error;
import org.springframework.security.oauth2.core.OAuth2TokenValidatorResult;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Component;

/**
 * 基于Redis的令牌验证器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@ConditionalOnClass(RedisContextHolder.class)
@ConditionalOnProperty(value = "oauth.redis.enabled", havingValue = "true", matchIfMissing = true)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RedisSessionValidator implements SessionValidator {
    private final OAuthProperties properties;

    @Override
    public OAuth2TokenValidatorResult validate(Jwt jwt) {
        if (OAuthContextHolder.isClient(jwt)) {
            return OAuth2TokenValidatorResult.success();
        }

        // 从Redis获取令牌信息
        Long id = OAuthContextHolder.getId(jwt);
        String terminal = String.valueOf(SessionContextHolder.getTerminal());
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        List<?> values = RedisHashHolder.get(
                template, OAuthContextHolder.key(id),
                terminal, OAuthContextHolder.SCOPE, OAuthContextHolder.MOBILE, OAuthContextHolder.REGISTERING
        );

        // 校验令牌有效性及应用范围
        String token = ObjectUtils.get(values, 0);
        String scope = OAuthContextHolder.getScope(jwt);
        if (StringUtils.isEmpty(token)) {
            throw new OAuth2AuthenticationException(new OAuth2Error(
                    "TOKEN_EXPIRED", ApplicationContextHolder.getMessage("Token.Expired"), null
            ));
        } else if (!Objects.equals(token, jwt.getTokenValue())) {
            // 判断令牌是否相同，如果不相同则表示令牌已经被重置（账号已在其他地方登录）
            throw new OAuth2AuthenticationException(new OAuth2Error(
                    "TOKEN_CHANGED", ApplicationContextHolder.getMessage("Token.Changed"), null
            ));
        } else if (this.properties.isScoped()) {
            // 判断应用范围是否相同，如果不同则表示应用范围已被重置，需要刷新令牌
            String real = ObjectUtils.get(values, 1);
            if (!Objects.equals(scope, real)) {
                String arg = StringUtils.ifEmpty(real, ApplicationContextHolder::getMessage);
                String message = ApplicationContextHolder.getMessage("Scope.Changed", new Object[]{arg});
                throw new OAuth2AuthenticationException(new OAuth2Error("SCOPE_CHANGED", message, null));
            }
        }

        // 更新会话主体信息
        String mobile = ObjectUtils.get(values, 2);
        Long registering = ObjectUtils.get(values, 3);
        long usage = RedisUserDailyHashHolder.increment(template, id, "usage");
        Using using = usage > 1 || registering == null ? Using.NORMAL :
                registering >= DateUtils.today().getTime() ? Using.NEWLY : Using.DAILY;
        SessionContextHolder.setSession(id, using, scope, mobile, OAuthContextHolder.getAuthorities(jwt));
        return OAuth2TokenValidatorResult.success();
    }
}
