package com.chelvc.framework.redis.context;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import lombok.NonNull;
import org.springframework.data.redis.core.RedisTemplate;

/**
 * Redis Hash结构当日数据操作工具类
 *
 * @author Woody
 * @date 2024/6/29
 */
public final class RedisUserDailyHashHolder {
    /**
     * 数据类型
     */
    public static final String TYPE = "user";

    private RedisUserDailyHashHolder() {
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param id  数据标识
     * @param key Hash键
     * @return Hash值
     */
    public static Object get(@NonNull Serializable id, @NonNull String key) {
        return RedisDailyHashHolder.get(TYPE, id, key);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param key      Hash键
     * @return Hash值
     */
    public static Object get(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                             @NonNull String key) {
        return RedisDailyHashHolder.get(template, TYPE, id, key);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param id   数据标识
     * @param keys Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull Serializable id, @NonNull String... keys) {
        return RedisDailyHashHolder.get(TYPE, id, keys);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param keys     Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull String... keys) {
        return RedisDailyHashHolder.get(template, TYPE, id, keys);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param id   数据标识
     * @param keys Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull Serializable id, @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.get(TYPE, id, keys);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param keys     Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.get(template, TYPE, id, keys);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param id    数据标识
     * @param array Hash键/值数组
     * @return true/false
     */
    public static boolean set(@NonNull Serializable id, @NonNull Object... array) {
        return RedisDailyHashHolder.set(TYPE, id, array);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param id  数据标识
     * @param map Hash键/值映射表
     * @return true/false
     */
    public static boolean set(@NonNull Serializable id, @NonNull Map<String, ?> map) {
        return RedisDailyHashHolder.set(TYPE, id, map);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param array    Hash键/值数组
     * @return true/false
     */
    public static boolean set(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Object... array) {
        return RedisDailyHashHolder.set(template, TYPE, id, array);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param map      Hash键/值映射表
     * @return true/false
     */
    public static boolean set(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Map<String, ?> map) {
        return RedisDailyHashHolder.set(template, TYPE, id, map);
    }

    /**
     * Redis Hash结构用户当日数字自增1
     *
     * @param id  数据标识
     * @param key Hash键
     * @return 增/减后数值
     */
    public static long increment(@NonNull Serializable id, @NonNull String key) {
        return RedisDailyHashHolder.increment(TYPE, id, key);
    }

    /**
     * Redis Hash结构用户当日数字自增/减，delta < 0 时自减
     *
     * @param id    数据标识
     * @param key   Hash键
     * @param delta 增/减值
     * @return 增/减后数值
     */
    public static long increment(@NonNull Serializable id, @NonNull String key, long delta) {
        return RedisDailyHashHolder.increment(TYPE, id, key, delta);
    }

    /**
     * Redis Hash结构用户当日数字自增1
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param key      Hash键
     * @return 增/减后数值
     */
    public static long increment(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                                 @NonNull String key) {
        return RedisDailyHashHolder.increment(template, TYPE, id, key);
    }

    /**
     * Redis Hash结构用户当日数字自增/减，delta < 0 时自减
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param key      Hash键
     * @param delta    增/减值
     * @return 增/减后数值
     */
    public static long increment(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                                 @NonNull String key, long delta) {
        return RedisDailyHashHolder.increment(template, TYPE, id, key, delta);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param id   数据标识
     * @param keys Hash键数组
     * @return 成功删除数量
     */
    public static long delete(Serializable id, @NonNull String... keys) {
        return RedisDailyHashHolder.delete(TYPE, id, keys);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param keys     Hash键数组
     * @return 成功删除数量
     */
    public static long delete(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull String... keys) {
        return RedisDailyHashHolder.delete(template, TYPE, id, keys);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param id   数据标识
     * @param keys Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull Serializable id, @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.delete(TYPE, id, keys);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       数据标识
     * @param keys     Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.delete(template, TYPE, id, keys);
    }
}
