package com.chelvc.framework.rocketmq.producer;

import java.util.Map;

import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.ProducerBuilder;
import org.apache.rocketmq.client.apis.producer.TransactionChecker;

/**
 * RocketMQ生产者工厂默认实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DefaultRocketMQProducerFactory implements RocketMQProducerFactory {
    private final ClientServiceProvider provider;
    private final ClientConfiguration configuration;
    private final Map<String, Producer> producers = Maps.newConcurrentMap();
    private final Map<String, Producer> transactionProducers = Maps.newConcurrentMap();

    public DefaultRocketMQProducerFactory(ClientServiceProvider provider, ClientConfiguration configuration) {
        this.provider = provider;
        this.configuration = configuration;
    }

    /**
     * 初始化消息生产者
     *
     * @param topic   消息主题
     * @param checker 事务检测回调接口
     * @return 消息生产者实例
     */
    private Producer initializeProducer(String topic, TransactionChecker checker) {
        ProducerBuilder builder = this.provider.newProducerBuilder().setClientConfiguration(this.configuration)
                .setTopics(topic);
        if (checker != null) {
            builder.setTransactionChecker(checker);
        }
        try {
            return builder.build();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 关闭消息生产者
     *
     * @param producer 消息生产者实例
     */
    private void shutdown(Producer producer) {
        if (producer != null) {
            try {
                producer.close();
            } catch (Throwable t) {
                log.warn("Message producer shutdown failed: {}", t.getMessage());
            }
        }
    }

    @Override
    public Producer getProducer(@NonNull String topic) {
        return this.producers.computeIfAbsent(topic, k -> this.initializeProducer(topic, null));
    }

    @Override
    public Producer getProducer(@NonNull String topic, @NonNull TransactionChecker checker) {
        return this.transactionProducers.computeIfAbsent(topic, k -> this.initializeProducer(topic, checker));
    }

    @Override
    public void destroy() throws Exception {
        this.producers.forEach((topic, producer) -> this.shutdown(producer));
        this.transactionProducers.forEach((topic, producer) -> this.shutdown(producer));
    }
}
