package com.chelvc.framework.security.config;

import java.util.Set;
import javax.servlet.Filter;

import com.chelvc.framework.base.interceptor.BufferedRequestInterceptor;
import com.chelvc.framework.common.annotation.Sensitive;
import com.chelvc.framework.security.context.DefaultSecurityCryptoContext;
import com.chelvc.framework.security.context.SecurityCryptoContext;
import com.chelvc.framework.security.interceptor.MethodSecurityExpression;
import com.chelvc.framework.security.serializer.SensitiveArraySerializer;
import com.chelvc.framework.security.serializer.SensitiveListSerializer;
import com.chelvc.framework.security.serializer.SensitiveSerializer;
import com.chelvc.framework.security.serializer.SensitiveSetSerializer;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.introspect.Annotated;
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
import com.fasterxml.jackson.databind.type.CollectionType;
import lombok.RequiredArgsConstructor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.security.access.expression.method.DefaultMethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionOperations;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.method.configuration.GlobalMethodSecurityConfiguration;
import org.springframework.security.core.Authentication;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;

/**
 * 安全配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@EnableGlobalMethodSecurity(prePostEnabled = true)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SecurityConfigurer extends GlobalMethodSecurityConfiguration implements ApplicationContextAware {
    private final SecurityProperties properties;

    @Override
    protected MethodSecurityExpressionHandler createExpressionHandler() {
        return new DefaultMethodSecurityExpressionHandler() {
            @Override
            protected MethodSecurityExpressionOperations createSecurityExpressionRoot(
                    Authentication authentication, MethodInvocation invocation) {
                MethodSecurityExpression root = new MethodSecurityExpression(authentication);
                root.setThis(invocation.getThis());
                root.setPermissionEvaluator(getPermissionEvaluator());
                root.setTrustResolver(getTrustResolver());
                root.setRoleHierarchy(getRoleHierarchy());
                root.setDefaultRolePrefix(getDefaultRolePrefix());
                return root;
            }
        };
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        // 设置敏感数据注解JSON序列化处理器
        if (!this.properties.getSensitive().isEnabled()) {
            return;
        }
        ObjectMapper mapper = applicationContext.getBean(ObjectMapper.class);
        mapper.setAnnotationIntrospector(new JacksonAnnotationIntrospector() {
            @Override
            public Object findSerializer(Annotated am) {
                if (!am.hasAnnotation(Sensitive.class)) {
                    return super.findSerializer(am);
                }

                JavaType type = am.getType();
                Class<?> raw = am.getRawType();
                if (raw == String.class) {
                    return SensitiveSerializer.class;
                } else if (raw.isArray() && raw.getComponentType() == String.class) {
                    return SensitiveArraySerializer.class;
                } else if (type instanceof CollectionType && type.getContentType().getRawClass() == String.class) {
                    if (Set.class.isAssignableFrom(type.getRawClass())) {
                        return SensitiveSetSerializer.class;
                    }
                    return SensitiveListSerializer.class;
                }
                return super.findSerializer(am);
            }
        });
    }

    @Bean
    @ConditionalOnMissingBean(SecurityCryptoContext.class)
    public SecurityCryptoContext securityCryptoContext() {
        return new DefaultSecurityCryptoContext(this.properties);
    }

    @Bean
    public FilterRegistrationBean<Filter> crossDomainAccessRegistration() {
        FilterRegistrationBean<Filter> registration = new FilterRegistrationBean<>();
        CorsConfiguration configuration = new CorsConfiguration();
        configuration.addAllowedOrigin("*");
        configuration.addAllowedHeader("*");
        configuration.addAllowedMethod("*");
        configuration.setAllowCredentials(true);
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);
        registration.setFilter(new CorsFilter(source));
        registration.setOrder(Ordered.HIGHEST_PRECEDENCE);
        return registration;
    }

    @Bean
    @ConditionalOnMissingBean(BufferedRequestInterceptor.class)
    public BufferedRequestInterceptor bufferedRequestInterceptor() {
        return new BufferedRequestInterceptor();
    }
}
