package com.chelvc.framework.security.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import javax.crypto.Cipher;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.base.interceptor.BufferedRequestWrapper;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdviceAdapter;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

/**
 * 接口参数加解密拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Order(2)
@ControllerAdvice
public class ControllerCryptoInterceptor extends RequestBodyAdviceAdapter implements ResponseBodyAdvice<Object> {
    /**
     * 获取目标方法@Crypto注解
     *
     * @param method 目标方法
     * @return @Crypto注解实例
     */
    private Crypto getCryptoAnnotation(MethodParameter method) {
        Crypto annotation = method.getMethodAnnotation(Crypto.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(Crypto.class);
        }
        return annotation;
    }

    /**
     * 数据加密
     *
     * @param original 原始数据
     * @return 数据密文
     */
    private String encrypt(Object original) {
        String plaintext = JacksonContextHolder.serialize(original);
        return SecurityContextHolder.getCryptoContext().encrypt(plaintext);
    }

    /**
     * 数据解密
     *
     * @param message 原始消息
     * @return 明文消息
     * @throws IOException I/O异常
     */
    private HttpInputMessage decrypt(HttpInputMessage message) throws IOException {
        byte[] ciphertext;
        try (InputStream input = message.getBody()) {
            if (input instanceof BufferedRequestWrapper.BufferedServletInputStream) {
                ciphertext = ((BufferedRequestWrapper.BufferedServletInputStream) input).decodeBase64();
            } else {
                ciphertext = Base64.decodeBase64(FileUtils.getBytes(input));
            }
        }
        Cipher cipher = SecurityContextHolder.getCryptoContext().getDecrypter();
        byte[] plaintext = CodecUtils.codec(cipher, ciphertext);
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return message.getHeaders();
            }

            @Override
            public InputStream getBody() throws IOException {
                return new ByteArrayInputStream(plaintext);
            }
        };
    }

    @Override
    public boolean supports(MethodParameter method, Type type, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.input();
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage message, MethodParameter method, Type type,
                                           Class<? extends HttpMessageConverter<?>> clazz) throws IOException {
        try {
            return this.decrypt(message);
        } catch (Exception e) {
            log.warn("Request parameter decrypt failed: {}", e.getMessage());
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Request.Invalid"));
        }
    }

    @Override
    public boolean supports(MethodParameter method, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.output();
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter method, MediaType media,
                                  Class<? extends HttpMessageConverter<?>> clazz,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        if (body == null) {
            return null;
        } else if (body instanceof Result) {
            Result<?> result = (Result<?>) body;
            if (result.getData() == null) {
                return result;
            }
            String ciphertext = this.encrypt(result.getData());
            return Result.of(result.getCode(), ciphertext, result.getMessage());
        }
        return this.encrypt(body);
    }
}
