package com.chelvc.framework.security.interceptor;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.common.annotation.Sensitive;
import com.chelvc.framework.security.context.SecurityContextHolder;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.Order;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

/**
 * 敏感数据响应拦截器实现
 *
 * @author Woody
 * @date 2024/6/22
 */
@Order(1)
@ControllerAdvice
@ConditionalOnProperty(value = "security.sensitive.enabled", havingValue = "true")
public class SensitiveResponseInterceptor implements ResponseBodyAdvice<Object> {
    /**
     * 数据加密
     *
     * @param method   请求方法
     * @param original 原始数据
     * @return 加密数据
     */
    @SuppressWarnings("unchecked")
    private Object encrypt(MethodParameter method, Object original) {
        if (original instanceof String) {
            return SecurityContextHolder.getCryptoContext().encrypt((String) original);
        } else if (original instanceof String[]) {
            return SecurityContextHolder.getCryptoContext().encrypt((String[]) original);
        } else if (original instanceof Collection) {
            Type type = Objects.requireNonNull(method.getMethod()).getGenericReturnType();
            if (type instanceof ParameterizedType) {
                Type raw = ((ParameterizedType) type).getRawType();
                Type arg = ((ParameterizedType) type).getActualTypeArguments()[0];
                if (raw == Set.class && arg == String.class) {
                    return SecurityContextHolder.getCryptoContext().encrypt((Set<String>) original);
                } else if (raw == List.class && arg == String.class) {
                    return SecurityContextHolder.getCryptoContext().encrypt((List<String>) original);
                }
            }
        }
        return original;
    }

    @Override
    public boolean supports(MethodParameter method, Class<? extends HttpMessageConverter<?>> clazz) {
        return Objects.nonNull(method.getMethod()) && method.hasMethodAnnotation(Sensitive.class);
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter method, MediaType media,
                                  Class<? extends HttpMessageConverter<?>> clazz,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        if (body == null) {
            return null;
        } else if (body instanceof Result) {
            Result<?> result = (Result<?>) body;
            if (result.getData() == null) {
                return result;
            }
            Object ciphertext = this.encrypt(method, result.getData());
            return Result.of(result.getCode(), ciphertext, result.getMessage());
        }
        return this.encrypt(method, body);
    }
}
